#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <unistd.h>
#include <stdlib.h>
#include <ctype.h>

#include "procstate.h"
#include "tests.h"

#define NBUSY	4
static pid_t pid[NBUSY];

int bunch_o_busy(int cpu);

int
main(void)
{
	int ret = 0;
	int i;

	/*
	 * make a bunch of busy threads, isolate 0, make sure they 
	 *     all end up on 1
	 */
	bunch_o_busy(0);

	sleep(1);
	if (sched_setprocstate(0, PROC_ISOLATED) < 0) {
		perror("setprocstate(RESTRICTED)\n");
		printf("fail\n");
		return 1;
	}

	/* signal the children to watch their cpu */
	for (i = 0; i < NBUSY; i++) {
		kill(pid[i], SIGHUP);
	}

	/* wait for them to exit - one failure is total failure */
	for (i = 0; i < NBUSY; i++) {
		int status;
		
		wait(&status);
		if (WEXITSTATUS(status)) {
			ret = 1;
			break;
		}
	}

	/* re-enabled the processor */
	sched_setprocstate(0, PROC_ENABLED);

	if (ret == 0)
		printf("pass\n");
	else
		printf("fail\n");

	return ret;
}

void dummy(int i)
{
	char buf[16];
	sprintf(buf, "%d", i);
}
	
/* when watch_cpu is set, start watching out CPU # */
static int watch_cpu;
void
hup(int sig)
{
	watch_cpu = 1;
}

void
do_busy_child(int cpu)
{
	int i;

	signal(SIGHUP, hup);
	while (!watch_cpu) {
		dummy(i++);
	}
	for (i = 0; i < 100; i++) {
		if (cur_cpu() == cpu)
			exit(1);
		usleep(1000);
	}
	exit(0);
}

int
bunch_o_busy(int cpu)
{
	int i;

	for (i = 0; i < NBUSY; i++) {
		pid[i] = fork();
		if (pid[i] == 0) { 
			/* child */
			do_busy_child(cpu);
		}
	}
	return 0;
}
